/*
  Motor Controller BV4603 1.2A, BV4604 1.4A
  Copyright (c) 2013 Jim Spence.  All right reserved.
  www.byvac.com - see terms and conditions for using hardware
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
  Release Notes:
  bv4619 - LCD controller with keypad interface that can so both serial and I2C

*/  

#include "Arduino.h"
#include "bv4603_4_I.h"

// ==============================================================
// I2C PRIVATE
// ==============================================================
// ==============================================================
// **************************************************************
// gets a 16 bit value from the i2c bus
// **************************************************************
int MOTOR::i2_16bit()
{
int rv;
  Wire.requestFrom(_i2adr, 2); // returns 2 bytes
  rv=Wire.read()*256; // high byte
  rv+=Wire.read(); // low byte
  return rv;
}

// =============================================================================
// I2C LCD display P018
// =============================================================================
// =============================================================================
// *****************************************************************************
// constructor is just a connection to NewSoftSerial
// *****************************************************************************
MOTOR::MOTOR(char i2cadr)
{
char j;
  _i2adr=i2cadr;  
  Wire.begin(); // join i2c bus
}


// =============================================================================
// =============================================================================
// CONFIGURATION SECTION
// =============================================================================
// *****************************************************************************
// write to eeprom
// see data sheet for address location and values. 
// Example set mode to serial / i2c
// motor.eeWrite(20,0)
// *****************************************************************************
void MOTOR::eeWrite(char address, char value)
{
    Wire.beginTransmission(_i2adr);
    Wire.write(0x91); //Write to eeprom
    Wire.write(address); // Address
    Wire.write(value); // value
    Wire.endTransmission();
}

// *****************************************************************************
// sets global power
// *****************************************************************************
void MOTOR::GlobalPower(char power)
{
    Wire.beginTransmission(_i2adr);
    Wire.write(4); // command
    Wire.write(power); // value
    Wire.endTransmission();
}

// =============================================================================
// =============================================================================
// DC MOTOR SECTION
// =============================================================================
// *****************************************************************************
// Sets power and direction for DC Motor 1
// power is 0 to 1023 and direction is either 0,1 or 2
// *****************************************************************************
void MOTOR::DC1(int power, char direction)
{
    Wire.beginTransmission(_i2adr);
    Wire.write(1); // command
    Wire.write((power >> 8) & 0xff); // value
    Wire.write(power & 0xff); // value
    Wire.write(direction);
    Wire.endTransmission();
}

// *****************************************************************************
// Sets power and direction for DC Motor 2
// power is 0 to 1023 and direction is either 0,1 or 2
// *****************************************************************************
void MOTOR::DC2(int power, char direction)
{
    Wire.beginTransmission(_i2adr);
    Wire.write(2); // command
    Wire.write((power >> 8) & 0xff); // value
    Wire.write(power & 0xff); // value
    Wire.write(direction);
    Wire.endTransmission();
}

// *****************************************************************************
// Sets power and direction for both DC motors at once, this avoids any problems
// of starting both motors together
// power is 0 to 1023 and direction is either 0,1 or 2
// *****************************************************************************
void MOTOR::DC3(int power1, char dir1, int power2, char dir2)
{
    Wire.beginTransmission(_i2adr);
    Wire.write(7); // command
    Wire.write((power1 >> 8) & 0xff); // value
    Wire.write(power1 & 0xff); // value
    Wire.write(dir1);
    Wire.write((power2 >> 8) & 0xff); // value
    Wire.write(power2 & 0xff); // value
    Wire.write(dir2);
    Wire.endTransmission();
}

// =============================================================================
// =============================================================================
// STEP SECTION
// =============================================================================
// *****************************************************************************
// Step to go, returns the number of steps left to go
// *****************************************************************************
long MOTOR::StepsToGo()
{
long tg;
    Wire.beginTransmission(_i2adr);
    Wire.write(3); // command
    Wire.endTransmission();
    Wire.requestFrom(_i2adr, 4); // returns 4 bytes
    tg =  Wire.read() << 24;
    tg += Wire.read() << 16;
    tg += Wire.read() << 8;
    tg += Wire.read();
    return tg;
}

// *****************************************************************************
// Step command
// #steps, speed, direction(1 or 0)
// *****************************************************************************
void MOTOR::Steps(long steps, int speed, char direction)
{
    Wire.beginTransmission(_i2adr);
    Wire.write(9); // command
    // # steps 4 bytes
    Wire.write((steps >> 24) & 0xff);
    Wire.write((steps >> 16) & 0xff);
    Wire.write((steps >> 8) & 0xff);
    Wire.write((steps >> 0) & 0xff);
    // speed 2 bytes
    Wire.write((speed >> 8) & 0xff);
    Wire.write((speed >> 0) & 0xff);
    // direction
    Wire.write(direction);
    Wire.endTransmission();
}

// *****************************************************************************
// Stop stepping
// *****************************************************************************
void MOTOR::Stop()
{
    Wire.beginTransmission(_i2adr);
    Wire.write(5); // command
    Wire.endTransmission();
}


